
from odoo.tests import TransactionCase, tagged
from odoo.exceptions import UserError
from odoo.addons.mail.tests.common import MailCommon
from odoo.addons.test_mail.data.test_mail_data import MAIL_TEMPLATE


@tagged("post_install", "-at_install")
class TestCrmTeamEducation(TransactionCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.Lead = cls.env["crm.lead"]
        cls.Team = cls.env["crm.team"]
        cls.country_moon = cls.env["res.country"].create({"code": "xm", "name": "Moon Station"})
        cls.crm_team_sale = cls.Team.create({"name": "Sale Team"})
        cls.crm_team_edu_be = cls.env.ref("openerp_crm.crm_team_edu_be")
        cls.crm_team_edu_hk = cls.env.ref("openerp_crm.crm_team_edu_hk")
        cls.crm_team_edu_us = cls.env.ref("openerp_crm.crm_team_edu_us")

    def test_get_edu_team_from_country(self):
        """Test that computation of CRM Education team depending on user source country."""
        edu_team = self.Team.get_edu_team("BE")
        self.assertEqual(
            edu_team, self.crm_team_edu_be,
            msg="Belgium education prospect should be assigned to BE Education Team"
        )
        edu_team = self.Team.get_edu_team("JP")
        self.assertEqual(
            edu_team, self.crm_team_edu_be,
            msg="Japan education prospect should be assigned to BE Education Team too"
        )

    def test_scaleup_allowed(self):
        # Lead on non-edu sales team
        lead = self.Lead.create({"name": "Support for teaching Odoo", "team_id": self.crm_team_sale.id})
        self.assertFalse(lead.allow_send_scaleup)

        # Lead on non-edu team but w/ scaleup mention
        lead = self.Lead.create({"name": "ScaleUp: Support for teaching Odoo", "team_id": self.crm_team_sale.id})
        self.assertTrue(
            lead.allow_send_scaleup,
            msg="We should allow sending ScaleUp if it's mentioned in the subject"
        )

        # Lead on edu team
        lead = self.Lead.create({"name": "Support for teaching Odoo", "team_id": self.crm_team_edu_be.id})
        self.assertTrue(
            lead.allow_send_scaleup,
            msg="We should always allow sending ScaleUp for an EDU team"
        )

    def test_scaleup_picking_creation(self):
        lang_es = self.env.ref('base.lang_es')
        lang_de = self.env.ref('base.lang_de')

        lead = self.Lead.create({"name": "ScaleUp: Support for teaching Odoo", "team_id": self.crm_team_sale.id})

        # test that unavailable scale-up language raises an error but does not crash
        lead.lang_id = lang_de
        with self.assertRaisesRegex(UserError, "No Scale-Up product available for language"):
            lead.action_prepare_picking_scaleup()

        # test that we can create the picking for available languages
        lead.lang_id = lang_es
        action = lead.action_prepare_picking_scaleup()
        self.assertTrue(action.get("res_id"))
        picking = self.env["stock.picking"].browse(action["res_id"])
        self.assertTrue(picking.state, "draft")
        self.assertEqual(
            picking.move_ids.product_id.product_tmpl_id,
            self.env.ref("openerp_delivery.product_business_game_product_template_es"),
            "The spanish version of the Scale-Up should have been added on the picking",
        )

@tagged("post_install", "-at_install")
class TestCrmMailGateway(MailCommon):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.Lead = cls.env["crm.lead"]
        cls.Team = cls.env["crm.team"]
        cls.Partner = cls.env["res.partner"]
        cls.crm_team_sale = cls.Team.create({"name": "Sale Team"})

    def test_crm_domain_typo(self):
        """Test automatic domain typo correction"""

        # Create a Lead without partner
        lead = self.Lead.create({"name": "TestTypo", "email_from": "typo1@gmai.com"})
        self.assertEqual(lead.email_from, "typo1@gmail.com")

        # Assigned a partner with a typo in email.
        partner = self.Partner.create({"name": "Partner1", "email": "partner1@hotmai.com"})
        lead.partner_id = partner
        self.assertEqual(lead.email_from, "partner1@hotmail.com")
        self.assertEqual(partner.email, "partner1@hotmai.com")  # still w/ the typo
        self.assertTrue(
            lead.partner_email_update,
            "there should be an indication that email will be synced to the partner"
        )

        # Create with both email & partner
        # (in that case both lead & partner should have typo corrected.)
        partner2 = self.Partner.create({"name": "Partner2", "email": "partner2@gmail.com"})
        lead2 = self.Lead.create({
            "name": "TestTypo2",
            "email_from": "typo2@gmail",
            "partner_id": partner2.id,
        })
        self.assertFalse(lead2.partner_email_update)
        self.assertEqual(lead2.email_from, "typo2@gmail.com")
        self.assertEqual(partner2.email, "typo2@gmail.com")

    def test_crm_mailgateway(self):
        if "openerp_enterprise" not in self.env["ir.module.module"]._installed():
            #  message_new() is relying on fields that are only defined in upstream
            # "openerp_enterprise" module, skip test if that module is not installed
            self.skipTest("openerp_enterprise module is not installed")

        self.crm_team_sale.alias_name = "be-sales"
        self.crm_team_sale.alias_domain_id.name = 'test.com'

        record = self.format_and_process(
            MAIL_TEMPLATE,
            "new-user@gmail.com", "be-sales@test.com",
            subject='Training for my school', target_model="crm.lead"
        )
        self.assertEqual(record._name, "crm.lead")
        self.assertEqual(record.team_id, self.crm_team_sale)
        self.assertIn("Ref: answer_mail", record.description)
        self.assertIn("Question: unknown_plan", record.description)

        partner_prof = self.env["res.partner"].create({
            "name": "Super Prof.",
            "email": "super-prof@myschoool.example.org",
            "subscription_oauth_scope": "trial",
            "plan": "plan_to_test",
        })
        record = self.format_and_process(
            MAIL_TEMPLATE,
            "super-prof@myschoool.example.org", "be-sales@test.com",
            subject='Teacher Odoo to my students', target_model="crm.lead"
        )
        self.assertEqual(record._name, "crm.lead")
        self.assertEqual(record.team_id, self.crm_team_sale)
        self.assertEqual(record.partner_id, partner_prof)
        self.assertIn("Ref: trial", record.description)
        self.assertIn("Question: plan_to_test", record.description)
